############################ Copyrights and license ############################
#                                                                              #
# Copyright 2021 karthik-kadajji <60779081+karthik-kadajji@users.noreply.github.com>#
# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

from . import Framework


class Organization(Framework.TestCase):
    def testIssue2030CreateProject(self):
        super().setUp()
        project = "ultratendency"
        self.user = self.g.get_user("karthik-kadajji-t")
        self.org = self.g.get_organization("testkarthik")
        self.org.create_project(project)
        project = self.org.get_projects()[0].name

        self.assertEqual("ultratendency", project)
